//-----------------------------------------------------------------------------
//  Copyright (C) 2014-2015 Thomas S. Ullrich
//
//  This file is part of "MineSweeper".
//
//  This file may be used under the terms of the GNU General Public License.
//  This project is free software; you can redistribute it and/or modify it
//  under the terms of the GNU General Public License.
//
//  Author: Thomas S. Ullrich
//  Last update: July 29, 2014
//-----------------------------------------------------------------------------
#include <QMessageBox>
#include "msCustomSizeDialog.h"
#include <iostream>

using namespace std;

msCustomSizeDialog::msCustomSizeDialog(QWidget *parent) : QDialog(parent, Qt::Sheet)
{
    setWindowTitle("MineSweeper - Custom Size");

    setModal(true);

    mVerticalLayout = new QVBoxLayout(this);
    mVerticalLayout->setSizeConstraint(QLayout::SetFixedSize);
    
    mFormLayout = new QFormLayout();
    mFormLayout->setFieldGrowthPolicy(QFormLayout::FieldsStayAtSizeHint);
    int numWidgetInForm = 0;
    
    //
    //   Number of rows label and spin box
    //
    mRowsLabel = new QLabel(this);
    mRowsSpinBox = new QSpinBox(this);
    mRowsSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
    mRowsSpinBox->setMinimum(2);
    mRowsSpinBox->setMaximum(999);  // adjusted later via setMaximumRows()
    mRowsSpinBox->setValue(20);     // will be set to current value when shown
    
    mFormLayout->setWidget(numWidgetInForm, QFormLayout::LabelRole, mRowsLabel);
    mFormLayout->setWidget(numWidgetInForm++, QFormLayout::FieldRole, mRowsSpinBox);
    
    //
    //   Number of columns label and spin box
    //
    mColumnsLabel = new QLabel(this);
    mColumnsSpinBox = new QSpinBox(this);
    mColumnsSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
    mColumnsSpinBox->setMinimum(2);
    mColumnsSpinBox->setMaximum(999);   // adjusted later via setMaximumColumns()
    mColumnsSpinBox->setValue(20);      // will be set to current value when shown

    mFormLayout->setWidget(numWidgetInForm, QFormLayout::LabelRole, mColumnsLabel);
    mFormLayout->setWidget(numWidgetInForm++, QFormLayout::FieldRole, mColumnsSpinBox);
    
    //
    //   Number of mines label and spin box
    //
    mMinesLabel = new QLabel(this);
    mRowsLabel->setText(QApplication::translate("Dialog", "Rows:", 0));
    mColumnsLabel->setText(QApplication::translate("Dialog", "Columns:", 0));
    mMinesLabel->setText(QApplication::translate("Dialog", "Mines:", 0));
    mMinesSpinBox = new QSpinBox(this);
    mMinesSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
    mMinesSpinBox->setMinimum(1);
    mMinesSpinBox->setMaximum(999);
    mMinesSpinBox->setValue(40);

    mFormLayout->setWidget(numWidgetInForm, QFormLayout::LabelRole, mMinesLabel);
    mFormLayout->setWidget(numWidgetInForm++, QFormLayout::FieldRole, mMinesSpinBox);
    mVerticalLayout->addLayout(mFormLayout);
    
    //
    //   Number of mines label and spin box
    //
    mSeparator = new QFrame(this);
    mSeparator->setFrameShape(QFrame::HLine);
    mSeparator->setFrameShadow(QFrame::Sunken);
    mVerticalLayout->addWidget(mSeparator);
    mButtonBox = new QDialogButtonBox(this);
    mButtonBox->setOrientation(Qt::Horizontal);
    mButtonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);
    mVerticalLayout->addWidget(mButtonBox);
    
    QObject::connect(mButtonBox, SIGNAL(accepted()), this, SLOT(accept()));
    QObject::connect(mButtonBox, SIGNAL(rejected()), this, SLOT(reject()));
}

msCustomSizeDialog::~msCustomSizeDialog() { /* no op */ }

void msCustomSizeDialog::accept()
{
    int nMines = mMinesSpinBox->value();
    int nRows  = mRowsSpinBox->value();
    int nColumns  = mColumnsSpinBox->value();
    if (nMines > 0.75*nRows*nColumns) {
        QMessageBox::critical(this, "Minesweeper",
                              tr("The number mines cannot exceed 3/4 of the number of cells (rows x columns).\n"
                                 "Reduce the number of mines or increase the game size."));
        return;
    }
    
    QDialog::accept();
}

int msCustomSizeDialog::rows() const {return mRowsSpinBox->value();}
int msCustomSizeDialog::columns() const {return mColumnsSpinBox->value();}
int msCustomSizeDialog::mines() const {return mMinesSpinBox->value();}

void msCustomSizeDialog::setRows(int val) {mRowsSpinBox->setValue(val);}
void msCustomSizeDialog::setColumns(int val) {mColumnsSpinBox->setValue(val);}
void msCustomSizeDialog::setMines(int val) {mMinesSpinBox->setValue(val);}

void msCustomSizeDialog::setMaximumRows(int val)
{
    mRowsSpinBox->setMaximum(val);
}

void msCustomSizeDialog::setMaximumColumns(int val)
{
    mColumnsSpinBox->setMaximum(val);
}
