//-----------------------------------------------------------------------------
//  Copyright (C) 2014-2015 Thomas S. Ullrich
//
//  This file is part of "MineSweeper".
//
//  This file may be used under the terms of the GNU General Public License.
//  This project is free software; you can redistribute it and/or modify it
//  under the terms of the GNU General Public License.
//
//  Author: Thomas S. Ullrich
//  Last update: July 29, 2014
//-----------------------------------------------------------------------------
#include "msHighscoreEntry.h"
#include <iostream>

using namespace std;

msHighscoreEntry::msHighscoreEntry()
{
    mTime = 0;
    mDurartion = 0;
//    mGameSize = 0;
    mIsLast = false;
}

msHighscoreEntry::msHighscoreEntry(QString name, time_t dat, time_t dur, msSettings::msGameSize gsize, bool isl)
{
    mTime = dat;
    mDurartion = dur;
    mGameSize = gsize;
    mPlayer = name;
    mIsLast = isl;
}

msHighscoreEntry::~msHighscoreEntry() {/* no op */}
    
QString msHighscoreEntry::name() const {return mPlayer;}

QString msHighscoreEntry::dateOfGame() const
{
    QString theDate = ctime(&mTime);
    theDate.remove('\n');
    return theDate;
}

time_t msHighscoreEntry::timeOfGame() const {return mTime;}

time_t msHighscoreEntry::durationOfGame() const {return mDurartion;}

msSettings::msGameSize msHighscoreEntry::gameSize() const {return mGameSize;}

bool msHighscoreEntry::isLast() const {return mIsLast;}

void msHighscoreEntry::setIsLast(bool val) {mIsLast = val;}

bool msHighscoreEntryCompare::operator()(msHighscoreEntry a, msHighscoreEntry b) const
{
    //
    //  operator < or return true if "a" is a higher score than "b"
    //
    if (a.durationOfGame() < b.durationOfGame()) { // shorter duration wins
        return true;
    }
    else if (a.durationOfGame() == b.durationOfGame()) {
        if (a.timeOfGame() < b.timeOfGame()) {     // older highscore wins
            return true;
        }
        else if (a.timeOfGame() == b.timeOfGame()) {
            if (a.name() < a.name()) {             // lexical sort
                return true;
            }
            else
                return false;
        }
        else
            return false;
    }
    else
        return false;
}

ostream& operator<<(ostream& os, const msHighscoreEntry& e)
{
    os << "time=" << e.timeOfGame() << ", duration=" << e.durationOfGame()
    << ", player=" << static_cast<const char*>(e.name().toLatin1())
    << ", size=" << static_cast<int>(e.gameSize())
    << ", last=" << e.isLast() << endl;
    return os;
}
