//-----------------------------------------------------------------------------
//  Copyright (C) 2014-2016 Thomas S. Ullrich
//
//  This file is part of "MineSweeper".
//
//  This file may be used under the terms of the GNU General Public License.
//  This project is free software; you can redistribute it and/or modify it
//  under the terms of the GNU General Public License.
//
//  Author: Thomas S. Ullrich
//  Last update: May 3, 2016
//-----------------------------------------------------------------------------
#ifndef msPreferenceDialog_h
#define msPreferenceDialog_h

class QGridLayout;
class QTabWidget;
class QWidget;
class QVBoxLayout;
class QHBoxLayout;
class QCheckBox;
class QGraphicsView;
class QGraphicsScene;
class QFormLayout;
class QLabel;
class QComboBox;
class QSpinBox;
class QSpacerItem;
class QDialogButtonBox;
class QGroupBox;
class QListWidget;
class QListWidgetItem;
class QSvgRenderer;
class msCell;

#include <QDialog>
#include <vector>
#include <map>
#include "msCell.h"

using namespace std;

class msPreferencesDialog : public QDialog
{
    Q_OBJECT
    
public:
    msPreferencesDialog(QWidget *parent = 0);
    ~msPreferencesDialog();
    
    void setMaxCellSize(unsigned int);
    
public slots:
    void accept();
    void reject();
    void backgroundColorComboActivated(int);
    void waterColorComboActivated(int);
    void cellSizeBoxActivated(int);

private:
    void addListItem(QListWidget*, vector<QListWidgetItem*>&, bool, const char*, const char* = 0);
    void createGraphicsScene();
    void setGraphicsSceneText(unsigned int, unsigned int, int);

private:
    QGraphicsView *mGraphicsView;
    QGraphicsScene *mGraphicsScene;
    QGraphicsRectItem *mSceneBackground;

    QGridLayout *mDialogGridLayout;
    QGridLayout *mGameOptionsTabGridLayout;
    QGridLayout *mCellAppearanceTabGridLayout;
    QGridLayout *mMinesGroupBoxGridLayout;
    QGridLayout *mFlagsGroupBoxGridLayout;
    QGridLayout *mPictoTabGridLayout;

    QSpacerItem *mVerticalSpacer1;
    QSpacerItem *mHorizontalSpacer1;
    QSpacerItem *mVerticalSpacer3;
    QSpacerItem *mVerticalSpacer2;

    QTabWidget *mTabWidget;
    
    QWidget *mGameOptionsTab;
    QWidget *mCellAppearanceTab;
    QWidget *mPictoTab;

    QCheckBox *mEasyStartupCheckBox;
    QCheckBox *mQuestionableFlagCheckBox;
    QCheckBox *mAllowOopsiesCheckBox;
    
    QLabel *mWaterColorLabel;
    QLabel *mBackgroundColorLabel;
    QLabel *mCellSizeLabel;
    
    QComboBox *mWaterColorComboBox;
    QComboBox *mBackgroundColorComboBox;

    QFormLayout *mFormLayout1;
    
    QSpinBox *mCellSizeSpinBox;
    
    QGroupBox *mMinesGroupBox;
    QGroupBox *mFlagsGroupBox;

    QListWidget *mMinesListWidget;
    QListWidget *mFlagsListWidget;
    
    QDialogButtonBox *mButtonBox;
    
    vector<vector<msCell*> > mCellArray;
    QColor mSelectedBackgroundColor;
    QColor mSelectedWaterColor;
    unsigned int mSelectedCellSize;
    
    vector<QListWidgetItem*> mMineItems;
    vector<QListWidgetItem*> mFlagItems;
    
    map<int, QSvgRenderer*> mNumberSvgRenderer;
};


#endif
