//-----------------------------------------------------------------------------
//  Copyright (C) 2014-2016 Thomas S. Ullrich
//
//  This file is part of "MineSweeper".
//
//  This file may be used under the terms of the GNU General Public License.
//  This project is free software; you can redistribute it and/or modify it
//  under the terms of the GNU General Public License.
//
//  Author: Thomas S. Ullrich
//  Last update: May 3, 2016
//-----------------------------------------------------------------------------
#ifndef msWindow_h
#define msWindow_h

#include <QMainWindow>
#include <QGraphicsTextItem>
#include <vector>
#include <ctime>
#include <map>
#include <random>

class QGraphicsView;
class QGraphicsScene;
class QAction;
class QMenu;
class QLabel;
class QTimer;
class QSvgRenderer;
class msCell;
class msCustomSizeDialog;
class msPreferencesDialog;
class msSettings;
class msHighscoreList;
class msAboutDialog;
class msOopsPixmap;
class msHelpBrowser;
class QGraphicsSvgItem;
class QPropertyAnimation;

using namespace std;

class msWindow : public QMainWindow
{
    Q_OBJECT

public:
    msWindow();
    ~msWindow();
    
protected:
    bool eventFilter(QObject*, QEvent*);
    void changeEvent(QEvent*);
    void positionGraphicsItem(QGraphicsSvgItem*, int, int, double = 1.);
    
public slots:
    void startGame();
    void updateClock();
    void setSmallSizeAndStartNew();
    void setMediumSizeAndStartNew();
    void setLargeSizeAndStartNew();
    void setCustomSize();
    void customSizeSelected(int);
    void showPreferences();
    void newPreferences(int);
    void showHighscores();
    void about();
    void help();
    void finish();
    void finishOopsAnimation();
    
private:
    unsigned int randomNumber(unsigned int b);
    void endGame(bool);
    bool isGameWon() const;
    void clearObvious(unsigned int, unsigned int);
    void createStatusBar();
    void createHelpBrowser();
    
    void startClock();
    void stopClock();
    void resetClock();
    
    void updateFlaggedMineCounter();
    void ensureBoardFitsOnScreen();
    void resizeMainWindow();
    
    void startOopsAnimation(const QPointF&);
    
private:
    bool           mGameIsOn;
    
    unsigned int   mNumberOfFlaggedMines;

    vector<vector<msCell*> > mCellArray;
    vector<msCell*> mNoMinesNoNeighborsCellArray;
    
    QGraphicsView   *mGraphicsView;
    QGraphicsScene  *mGraphicsScene;
    
    QSvgRenderer *mMineSvgRenderer;
    QSvgRenderer *mFlagSvgRenderer;
    QSvgRenderer *mTextSvgRenderer;
    QSvgRenderer *mTryOutSvgRenderer;
    map<int, QSvgRenderer*> mNumberSvgRenderer;
    
    QMenu *mGameMenu;
    QMenu *mGameSizeMenu;
    QMenu *mHelpMenu;
    
    QAction *mNewGameAction;
    QAction *mSmallSizeAction;
    QAction *mMediumSizeAction;
    QAction *mLargeSizeAction;
    QAction *mCustomSizeAction;
    QAction *mPreferencesAction; 
    QAction *mHighscoreAction;
    QAction *mAboutAction;
    QAction *mHelpAction;
    QAction *mFinishAction;

    time_t          mStartTime;
    time_t          mEndTime;
    time_t          mInactiveTime;
    
    QLabel          *mFlagCountLabel;
    QLabel          *mMessageLabel;
    QLabel          *mClockLabel;
    QLabel          *mCheatLabel;
    QTimer          *mClockUpdateTimer;
    
    msCustomSizeDialog   *mCustomSizeDialog;
    msPreferencesDialog  *mPreferencesDialog;
    msHighscoreList      *mHighscoreList;
    msAboutDialog        *mAboutDialog;
    msHelpBrowser        *mHelpBrowser;
    
    msSettings *mSettings;
    
    default_random_engine mRandomEngine;
    
    unsigned int mOopsiesLeft;
    QPropertyAnimation *mAnimation;
    msOopsPixmap *mOopsPixmap;
};

#endif

